<?php

namespace Drupal\mappi_lane\Plugin\Field\FieldWidget;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Driver\Exception\Exception;
use Drupal\mappi_lane\MappiApiClient;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'viewfield' widget.
 *
 * @FieldWidget (
 *    id = "mappi_default_widget",
 *    label = @Translation("Mappi default"),
 *    field_types = {
 *      "mappi"
 *   }
 * )
 */
class MappiDefaultWidget extends WidgetBase {

  protected MappiApiClient $mappiApiClient;

  public function __construct(
    $plugin_id,
    $plugin_definition,
    FieldDefinitionInterface $field_definition,
    array $settings,
    array $third_party_settings,
    MappiApiClient $mappi_api_client
  ) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings);

    $this->mappiApiClient = $mappi_api_client;
  }

  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['third_party_settings'],
      $container->get('mappi_lane.api_client')
    );
  }

  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $map_id = $items[$delta]->value ?? NULL;

    $element['value'] = [
      '#title' => $this->t('Mappi kaarten'),
      '#type' => 'select',
      '#options' => $this->getMapOptions(),
      '#default_value' => $map_id,
    ];
    $element['update_trigger'] = [
      '#type' => 'submit',
      '#value' => $this->t('Select map'),
      '#submit' => [
        [static::class, 'submitSelectMap'],
      ],
      '#ajax' => [
        'callback' => [static::class, 'updateSettings'],
        'wrapper' => 'edit-categories',
        'progress' => [
          'type' => 'throbber',
          'message' => $this->t('Verifying entry...'),
        ],
      ],
    ];
    $element['categories'] = [
      '#title' => $this->t('Categories'),
      '#type' => 'checkboxes',
      '#options' => $this->getMapCategoryOptions($map_id, $this->getMapOptions()),
      '#default_value' => json_decode($items->settings, TRUE)['categories'] ?? NULL,
      '#prefix' => '<div id="edit-categories">',
      '#suffix' => '</div>',
    ];

    return $element;
  }

  public function massageFormValues(array $values, array $form, FormStateInterface $form_state) {
    $values = parent::massageFormValues($values, $form, $form_state);

    foreach ($values as $delta => $value) {
      $settings = [
        'categories' => $value['categories'],
      ];

      $values[$delta]['settings'] = json_encode($settings);
    }

    return $values;
  }

  public static function submitSelectMap(array &$form, FormStateInterface $form_state) {
    $form_state->setRebuild();
  }

  public static function updateSettings(array &$form, FormStateInterface $form_state) {
    $button = $form_state->getTriggeringElement();

    // Go one level up in the form, to the widget container.
    $widget = NestedArray::getValue($form, array_slice($button['#array_parents'], 0, -1));

    return $widget['categories'];
  }

  protected function getMapOptions(): array {
    $mapOptions = [];
    $maps = $this->mappiApiClient->getMaps($this->getFieldSetting('customer_base_url'));

    foreach ($maps as $map) {
      $mapOptions[$map['id']] = sprintf('%s (%s)', $map['title'], $map['url']);
    }

    return $mapOptions;
  }

  protected function getMapCategoryOptions($map_id, $map_options): array {
    $categoryOptions = [];
    if (!array_key_exists($map_id, $map_options)) {
      return [];
    }

    $mapCategories = json_decode($this->mappiApiClient->getMapSettings(
      $this->getFieldSetting('customer_base_url'),
      $map_id
    )['categories'], TRUE);

      foreach ($mapCategories as $category) {
        if (!array_key_exists('children', $category)) {
          $categoryOptions[$category['id']] = $category['title'];
          continue;
        }

        foreach ($category['children'] as $child) {
          $categoryOptions[$child['id']] = $category['title'] . ' > ' . ucfirst($child['title']);
        }
      }

      return $categoryOptions;
  }

}
